#include "glut_import.h"
#include <math.h>
#include <stdio.h>
#include <string.h>

/*********************************************************************
 * MENU TYPES                                                        *
 *********************************************************************/

typedef struct {
  void (* OnExecute)(void);
} statictext;

typedef struct {
  int Checked;
  void (* OnChange)(int checked);
} checkbox;

typedef struct {
  int ItemCount,ItemIndex;
  char **Items;
  void (* OnChange)(int index);
} listbox;

typedef struct {
  int Min,Max,Position,Increment;
  char CurrentCaption[256];
  void (* OnChange)(int position);
} itrackbar;

typedef struct {
  float Min,Max,Position,Increment;
  char CurrentCaption[256];
  void (* OnChange)(float position);
} ftrackbar;

typedef union {
  statictext *StaticText;
  checkbox *CheckBox;
  listbox *ListBox;
  itrackbar *ITrackBar;
  ftrackbar *FTrackBar;
} itemdata;

typedef struct {
  int Class,Enabled;
  char *Caption;
  itemdata Data;
} menuitem;

typedef struct {
  char *Caption;
  int ItemCount,ItemIndex;
  menuitem **Items;
} menupage;

typedef struct {
  int Active,PageCount,PageIndex;
  menupage **Pages;
} menu;

/*********************************************************************
 * MENU THEME                                                        *
 *********************************************************************/

struct {
  unsigned char BackGroundColor[4];
  unsigned char BorderColor[4];
  float BorderWidth;
  unsigned char ActiveTextColor[4];
  unsigned char InactiveTextColor[4];
  int RoundPageCorners[4];
} Theme={
  {50,70,50,100},
  {255,255,255,200},
  1.5,
  {255,200,80,255},
  {255,255,255,130},
  {0,1,1,1}
};

/*********************************************************************
 * MENU CONSTANTS                                                    *
 *********************************************************************/

#define ROUND_RECT_OFFSET 15
#define ROUND_RECT_QUALITY 25

#define TRACKBAR_FREQUENCY 3

#define PI 3.141592653589793

/*********************************************************************
 * MENU CLASSES                                                      *
 *********************************************************************/

#define CLASS_SEPARATOR     0x01
#define CLASS_STATICTEXT    0x02
#define CLASS_CHECKBOX      0x04
#define CLASS_LISTBOX       0x08
#define CLASS_ITRACKBAR     0x10
#define CLASS_FTRACKBAR     0x20

/*********************************************************************
 * MENU BUILD MACRO                                                  *
 *********************************************************************/

#define SEPARATOR(_NAME)                                                                 \
  static menuitem _NAME={CLASS_SEPARATOR,0,"",{NULL}}

#define STATICTEXT(_NAME,_CAPTION,_ENABLED,_ONEXECUTE)                                   \
  static statictext _NAME##_DATA={_ONEXECUTE};                                           \
  static menuitem _NAME={CLASS_STATICTEXT,_ENABLED,_CAPTION,{&_NAME##_DATA}}

#define CHECKBOX(_NAME,_CAPTION,_CHECKED,_ONCHANGE)                                      \
  static checkbox _NAME##_DATA={_CHECKED,_ONCHANGE};                                     \
  static menuitem _NAME={CLASS_CHECKBOX,1,_CAPTION,{(statictext *) &_NAME##_DATA}}

#define LISTBOX(_NAME,_CAPTION,_INDEX,_ITEMS,_ONCHANGE)                                  \
  static listbox _NAME##_DATA={sizeof(_ITEMS)/sizeof(*_ITEMS),_INDEX,_ITEMS,_ONCHANGE};  \
  static menuitem _NAME={CLASS_LISTBOX,1,_CAPTION,{(statictext *) &_NAME##_DATA}}

#define ITRACKBAR(_NAME,_CAPTION,_MIN,_MAX,_POSITION,_INCREMENT,_ONCHANGE)               \
  static itrackbar _NAME##_DATA={_MIN,_MAX,_POSITION,_INCREMENT,_CAPTION,_ONCHANGE};     \
  static menuitem _NAME={CLASS_ITRACKBAR,1,_CAPTION,{(statictext *) &_NAME##_DATA}}

#define FTRACKBAR(_NAME,_CAPTION,_MIN,_MAX,_POSITION,_INCREMENT,_ONCHANGE)               \
  static ftrackbar _NAME##_DATA={_MIN,_MAX,_POSITION,_INCREMENT,_CAPTION,_ONCHANGE};     \
  static menuitem _NAME={CLASS_FTRACKBAR,1,_CAPTION,{(statictext *) &_NAME##_DATA}}

#define MENUPAGE(_NAME,_CAPTION,_ITEMS)                                                  \
  static menupage _NAME={_CAPTION,sizeof(_ITEMS)/sizeof(*_ITEMS),0,(menuitem **) &_ITEMS}

#define MENU(_NAME,_PAGES)                                                               \
  static menu _NAME={0,sizeof(_PAGES)/sizeof(*_PAGES),0,(menupage **) &_PAGES}

/*********************************************************************
 * DRAWING UTILITIES                                                 *
 *********************************************************************/
 
int strWidth(void *font,char *str){
  int w=0;
  char c;
  while (c=*(str++))
    w+=glutBitmapWidth(font,c);
  return w;
}

void strPaint(void *font,char *str,float x,float y){
  char c;
  glRasterPos2f(x,y);
  while (c=*(str++))
    glutBitmapCharacter(font,c);
}

void paintArrowShape(int x,int y,int size){
  int h=abs(size);
  glVertex2f(x+0.30*size,y+0.30*h);
  glVertex2f(x+0.30*size,y+1.00*h);
  glVertex2f(x+0.65*size,y+0.65*h);
}

void paintLeftArrow(int size,int active,int x,int y){
  if (active)
    glColor3ubv(Theme.ActiveTextColor);
  else
    glColor3ubv(Theme.InactiveTextColor);
  glBegin(GL_TRIANGLES);
  paintArrowShape(x+size,y,-size);
  glEnd();
}

void paintRightArrow(int size,int active,int x,int y){
  if (active)
    glColor3ubv(Theme.ActiveTextColor);
  else
    glColor3ubv(Theme.InactiveTextColor);
  glBegin(GL_TRIANGLES);
  paintArrowShape(x,y,size);
  glEnd();
}

void paintCheckBox(int size,int checked,int active,int x,int y){
  if (active)
    glColor3ubv(Theme.ActiveTextColor);
  else
    glColor3ubv(Theme.InactiveTextColor);
  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.1*size,y+0.2*size);
  glVertex2f(x+0.1*size,y+1.0*size);
  glVertex2f(x+0.9*size,y+1.0*size);
  glVertex2f(x+0.9*size,y+0.2*size);
  glEnd();
  if (checked){
    glBegin(GL_LINE_STRIP);
    glVertex2f(x+0.2*size,y+0.6*size);
    glVertex2f(x+0.3*size,y+0.6*size);
    glVertex2f(x+0.5*size,y+0.9*size);
    glVertex2f(x+0.8*size,y+0.3*size);
    glEnd();
  }
}

void paintTrackBar(int size,int active,float position,float x,int y,float width){
  int i,l;
  x+=0.3*size;
  width-=0.4*size;
  l=TRACKBAR_FREQUENCY*(position*width/TRACKBAR_FREQUENCY);
  glBegin(GL_LINES);
  for (i=TRACKBAR_FREQUENCY/2;i<=l;i+=TRACKBAR_FREQUENCY){
    if (active){
      float r=0.5*sqrt(i/width);
      glColor3f(1-r,0.5+r,0.1);
    }
    glVertex2f(x+i,y+0.2*size);
    glVertex2f(x+i,y+1.0*size);
  }
  glEnd();
  if (active)
    glColor3ubv(Theme.ActiveTextColor);
  else
    glColor3ubv(Theme.InactiveTextColor);
  glBegin(GL_LINE_LOOP);
  glVertex2f(x,y+0.2*size);
  glVertex2f(x+width,y+0.2*size);
  glVertex2f(x+width,y+1.0*size);
  glVertex2f(x,y+1.0*size);
  glEnd();
}

void paintMenuPageShape(int x,int y,int width,int height){
  int i,j,T[4][2]={{0,0},{1,0},{1,1},{0,1}};
  for (i=0;i<4;i++){
    if (Theme.RoundPageCorners[i]){
      int u=x+T[i][0]*width+(1-2*T[i][0])*ROUND_RECT_OFFSET;
      int v=y+T[i][1]*height+(1-2*T[i][1])*ROUND_RECT_OFFSET;
      for (j=0;j<=ROUND_RECT_QUALITY;j++){
        float o=0.5*PI*(i+(float) j/ROUND_RECT_QUALITY);
        glVertex2f(u-ROUND_RECT_OFFSET*cos(o),v-ROUND_RECT_OFFSET*sin(o));
      }
    } else
      glVertex2f(x+T[i][0]*width,y+T[i][1]*height);
  }
}

void paintMenuPageBackground(int FontSize,int x,int y,int width,int height){
  glColor4ubv(Theme.BackGroundColor);
  glBegin(GL_POLYGON);
  paintMenuPageShape(x,y,width,height);
  glEnd();
  glColor4ubv(Theme.BorderColor);
  glLineWidth(Theme.BorderWidth);
  glBegin(GL_LINE_LOOP);
  paintMenuPageShape(x,y,width,height);
  glEnd();
  glBegin(GL_LINES);
  glVertex2f(x,y+1.3*FontSize);
  glVertex2f(x+width,y+1.3*FontSize);
  glEnd();
}

/*********************************************************************
 * MENU PAINTING                                                     *
 *********************************************************************/

int menuItemWidth(void *Font,int FontSize,menuitem *Item,int PageWidth){
  int w=0;
  switch (Item->Class){
    case CLASS_STATICTEXT:
      w=strWidth(Font,Item->Caption);
      break;
    case CLASS_CHECKBOX:
      w=strWidth(Font,Item->Caption)+FontSize;
      break;
    case CLASS_LISTBOX:
      w=strWidth(Font,Item->Caption)+strWidth(Font,Item->Data.ListBox->Items[Item->Data.ListBox->ItemIndex])+2*FontSize;
      break;
    case CLASS_ITRACKBAR:
      sprintf(Item->Data.ITrackBar->CurrentCaption,Item->Caption,Item->Data.ITrackBar->Position);
      w=strWidth(Font,Item->Data.ITrackBar->CurrentCaption)+6*FontSize;
      break;
    case CLASS_FTRACKBAR:
      sprintf(Item->Data.FTrackBar->CurrentCaption,Item->Caption,Item->Data.FTrackBar->Position);
      w=strWidth(Font,Item->Data.FTrackBar->CurrentCaption)+6*FontSize;
      break;
  }
  return w;
}

void paintMenuItem(void *Font,int FontSize,menuitem *Item,int Active,int x,int y,int Width){
  int l;
  float r;
  if (Active)
    glColor4ubv(Theme.ActiveTextColor);
  else
    glColor4ubv(Theme.InactiveTextColor);
  switch (Item->Class){
    case CLASS_SEPARATOR:
      glColor4ubv(Theme.BorderColor);
      glLineWidth(Theme.BorderWidth);
      glBegin(GL_LINES);
      glVertex2f(x-ROUND_RECT_OFFSET,y-0.4*FontSize);
      glVertex2f(x+Width+ROUND_RECT_OFFSET,y-0.4*FontSize);
      glEnd();
      break;
    case CLASS_STATICTEXT:
      strPaint(Font,Item->Caption,x,y);
      break;
    case CLASS_CHECKBOX:
      strPaint(Font,Item->Caption,x,y);
      paintCheckBox(FontSize,Item->Data.CheckBox->Checked,Active,x+Width-FontSize,y-FontSize);
      break;
    case CLASS_LISTBOX:
      strPaint(Font,Item->Caption,x,y);
      l=strWidth(Font,Item->Data.ListBox->Items[Item->Data.ListBox->ItemIndex]);
      strPaint(Font,Item->Data.ListBox->Items[Item->Data.ListBox->ItemIndex],x+Width-l-FontSize,y);
      if (Item->Data.ListBox->ItemIndex>0)
        paintLeftArrow(FontSize,Active,x+Width-2*FontSize-l,y-FontSize);
      if (Item->Data.ListBox->ItemIndex<Item->Data.ListBox->ItemCount-1)
        paintRightArrow(FontSize,Active,x+Width-FontSize,y-FontSize);
      break;
    case CLASS_ITRACKBAR:
      strPaint(Font,Item->Data.ITrackBar->CurrentCaption,x,y);
      r=(float) (Item->Data.ITrackBar->Position-Item->Data.ITrackBar->Min)/(Item->Data.ITrackBar->Max-Item->Data.ITrackBar->Min);
      paintTrackBar(FontSize,Active,r,x+Width-6*FontSize,y-FontSize,6*FontSize);
      break;
    case CLASS_FTRACKBAR:
      strPaint(Font,Item->Data.FTrackBar->CurrentCaption,x,y);
      r=(Item->Data.FTrackBar->Position-Item->Data.FTrackBar->Min)/(Item->Data.FTrackBar->Max-Item->Data.FTrackBar->Min);
      paintTrackBar(FontSize,Active,r,x+Width-6*FontSize,y-FontSize,6*FontSize);
      break;
  }
}

void paintMenuPage(menupage *Page,int Width,int Height,int CanLeft,int CanRight){
  int i,j,k=3,l,w,x,y,h,H[3]={12,14,20};
  void *f,*F[3]={GLUT_BITMAP_HELVETICA_10,GLUT_BITMAP_HELVETICA_12,GLUT_BITMAP_HELVETICA_18};
  do {
    k--;
    f=F[k];
    h=H[k];
    w=l=strWidth(f,Page->Caption)+2*h;
    for (i=0;i<Page->ItemCount;i++)
      if ((j=menuItemWidth(f,h,Page->Items[i],Width-2*ROUND_RECT_OFFSET))>w)
        w=j;
  } while (k && (w+2*ROUND_RECT_OFFSET>Width || h*(Page->ItemCount+1)+2*ROUND_RECT_OFFSET>Height));
  x=(Width-w-2*ROUND_RECT_OFFSET)/2;
  if (x<0)
    x=0;
  y=(Height-h*(Page->ItemCount+1)-2*ROUND_RECT_OFFSET)/2;
  if (y<0)
    y=0;
  paintMenuPageBackground(h,x,y,w+2*ROUND_RECT_OFFSET,h*(Page->ItemCount+1)+2*ROUND_RECT_OFFSET);
  for (i=0;i<Page->ItemCount;i++)
    paintMenuItem(f,h,Page->Items[i],i+1==Page->ItemIndex,x+ROUND_RECT_OFFSET,y+ROUND_RECT_OFFSET+h*(i+2),w);
  if (Page->ItemIndex){
    i=0;
    glColor4ubv(Theme.InactiveTextColor);
  } else {
    i=1;
    glColor4ubv(Theme.ActiveTextColor);
  }
  strPaint(f,Page->Caption,x+ROUND_RECT_OFFSET+h+0.5*(w-l),y+h);
  if (CanLeft)
    paintLeftArrow(h,i,x+ROUND_RECT_OFFSET,y);
  if (CanRight)
    paintRightArrow(h,i,x+ROUND_RECT_OFFSET+w-h,y);
}

/*********************************************************************
 * MENU LOADING                                                      *
 *********************************************************************/
 
menuitem *findMenuItem(menu *Menu,char *Page,char *Caption){
  int i,j;
  for (i=0;i<Menu->PageCount;i++)
    if (!strcmp(Menu->Pages[i]->Caption,Page)){
      menupage *Page=Menu->Pages[i];
      for (j=0;j<Page->ItemCount;j++)
        if (!strcmp(Page->Items[j]->Caption,Caption))
          return Page->Items[j];
    }
  return NULL;
}
 
 void menuInit(menu *Menu){
  int i,j;
  for (i=0;i<Menu->PageCount;i++)
    for (j=0;j<Menu->Pages[i]->ItemCount;j++){
      menuitem *Item=Menu->Pages[i]->Items[j];
      switch(Item->Class){
        case CLASS_STATICTEXT:
          break;
        case CLASS_CHECKBOX:
          if (Item->Data.CheckBox->OnChange)
            Item->Data.CheckBox->OnChange(Item->Data.CheckBox->Checked);
          break;
        case CLASS_LISTBOX:
          if (Item->Data.ListBox->OnChange)
            Item->Data.ListBox->OnChange(Item->Data.ListBox->ItemIndex);
          break;
        case CLASS_ITRACKBAR:
          if (Item->Data.ITrackBar->OnChange)
            Item->Data.ITrackBar->OnChange(Item->Data.ITrackBar->Position);
          break;
        case CLASS_FTRACKBAR:
          if (Item->Data.FTrackBar->OnChange)
            Item->Data.FTrackBar->OnChange(Item->Data.FTrackBar->Position);
          break;
      }
    }
}

void menuLoad(menu *Menu,FILE *f){
  char Comment[256],Section[256]={0},Key[256],Value[256];
  while (!feof(f)){
    if (!fscanf(f," ;%255[^\n]",Comment) && fscanf(f," [%255[^]]]",Section)!=1 && *Section && fscanf(f," %255[^=\n]=%255[^\n]",Key,Value)==2){
      /*printf("Section=\"%s\" Key=\"%s\" Value=\"%s\"\n",Section,Key,Value);*/
      menuitem *Item=findMenuItem(Menu,Section,Key);
      if (Item){
        int i;
        switch(Item->Class){
          case CLASS_STATICTEXT:
            break;
          case CLASS_CHECKBOX:
            sscanf(Value,"%d",&i);
            Item->Data.CheckBox->Checked=i?1:0;
            break;
          case CLASS_LISTBOX:
            sscanf(Value,"%d",&Item->Data.ListBox->ItemIndex);
            if (Item->Data.ListBox->ItemIndex<0)
              Item->Data.ListBox->ItemIndex=0;
            if (Item->Data.ListBox->ItemIndex>=Item->Data.ListBox->ItemCount)
              Item->Data.ListBox->ItemIndex=Item->Data.ListBox->ItemCount-1;
            break;
          case CLASS_ITRACKBAR:
            sscanf(Value,"%d",&Item->Data.ITrackBar->Position);
            if (Item->Data.ITrackBar->Position<Item->Data.ITrackBar->Min)
              Item->Data.ITrackBar->Position=Item->Data.ITrackBar->Min;
            if (Item->Data.ITrackBar->Position>Item->Data.ITrackBar->Max)
              Item->Data.ITrackBar->Position=Item->Data.ITrackBar->Max;
            break;
          case CLASS_FTRACKBAR:
            sscanf(Value,"%f",&Item->Data.FTrackBar->Position);
            if (Item->Data.FTrackBar->Position<Item->Data.FTrackBar->Min)
              Item->Data.FTrackBar->Position=Item->Data.FTrackBar->Min;
            if (Item->Data.FTrackBar->Position>Item->Data.FTrackBar->Max)
              Item->Data.FTrackBar->Position=Item->Data.FTrackBar->Max;
            break;
        }
      }
    }
    fscanf(f,"%*[^\n]");
    fgetc(f);
  }
}

void menuSave(menu *Menu,FILE *f){
  int i,j;
  for (i=0;i<Menu->PageCount;i++){
    fprintf(f,"[%s]\n",Menu->Pages[i]->Caption);
    for (j=0;j<Menu->Pages[i]->ItemCount;j++){
      menuitem *Item=Menu->Pages[i]->Items[j];
      switch(Item->Class){
        case CLASS_STATICTEXT:
          break;
        case CLASS_CHECKBOX:
          fprintf(f,"%s=%d\n",Item->Caption,Item->Data.CheckBox->Checked);
          break;
        case CLASS_LISTBOX:
          fprintf(f,"%s=%d\n",Item->Caption,Item->Data.ListBox->ItemIndex);
          break;
        case CLASS_ITRACKBAR:
          fprintf(f,"%s=%d\n",Item->Caption,Item->Data.ITrackBar->Position);
          break;
        case CLASS_FTRACKBAR:
          fprintf(f,"%s=%f\n",Item->Caption,Item->Data.FTrackBar->Position);
          break;
      }
    }
    fprintf(f,"\n");
  }
}

/*********************************************************************
 * MENU KEYS                                                         *
 *********************************************************************/

void menuItemKeyboard(menuitem *Item,unsigned char Key){
  switch (Item->Class){
    case CLASS_STATICTEXT:
      switch (Key){
        case ' ':
        case 13:
          if (Item->Data.StaticText->OnExecute)
            Item->Data.StaticText->OnExecute();
          break;
      }
      break;
    case CLASS_CHECKBOX:
      switch (Key){
        case ' ':
        case 13:
          Item->Data.CheckBox->Checked=!Item->Data.CheckBox->Checked;
          if (Item->Data.CheckBox->OnChange)
            Item->Data.CheckBox->OnChange(Item->Data.CheckBox->Checked);
          break;
      }
      break;
    case CLASS_LISTBOX:
      switch (Key){
        case ' ':
        case 13:
          Item->Data.ListBox->ItemIndex=(Item->Data.ListBox->ItemIndex+1) % Item->Data.ListBox->ItemCount;
          if (Item->Data.ListBox->OnChange)
            Item->Data.ListBox->OnChange(Item->Data.ListBox->ItemIndex);
          break;
      }
      break;
    case CLASS_ITRACKBAR:
      break;
    case CLASS_FTRACKBAR:
      break;
  }
}

void menuItemSpecial(menuitem *Item,int Key){
  int u=1;
  if (glutGetModifiers())
    u=10;
  switch (Item->Class){
    case CLASS_STATICTEXT:
      break;
    case CLASS_CHECKBOX:
      break;
    case CLASS_LISTBOX:
      switch (Key){
        case GLUT_KEY_LEFT:
          if (Item->Data.ListBox->ItemIndex){
            Item->Data.ListBox->ItemIndex--;
            if (Item->Data.ListBox->OnChange)
              Item->Data.ListBox->OnChange(Item->Data.ListBox->ItemIndex);
          }
          break;
        case GLUT_KEY_RIGHT:
          if (Item->Data.ListBox->ItemIndex<Item->Data.ListBox->ItemCount-1){
            Item->Data.ListBox->ItemIndex++;
            if (Item->Data.ListBox->OnChange)
              Item->Data.ListBox->OnChange(Item->Data.ListBox->ItemIndex);
          }
          break;
      }
      break;
    case CLASS_ITRACKBAR:
      switch(Key){
        case GLUT_KEY_PAGE_DOWN:
          u*=10;
        case GLUT_KEY_LEFT:
          Item->Data.ITrackBar->Position-=u*Item->Data.ITrackBar->Increment;
          if (Item->Data.ITrackBar->Position<Item->Data.ITrackBar->Min)
            Item->Data.ITrackBar->Position=Item->Data.ITrackBar->Min;
          if (Item->Data.ITrackBar->OnChange)
            Item->Data.ITrackBar->OnChange(Item->Data.ITrackBar->Position);
          break;
        case GLUT_KEY_PAGE_UP:
          u*=10;
        case GLUT_KEY_RIGHT:
          Item->Data.ITrackBar->Position+=u*Item->Data.ITrackBar->Increment;
          if (Item->Data.ITrackBar->Position>Item->Data.ITrackBar->Max)
            Item->Data.ITrackBar->Position=Item->Data.ITrackBar->Max;
          if (Item->Data.ITrackBar->OnChange)
            Item->Data.ITrackBar->OnChange(Item->Data.ITrackBar->Position);
          break;
      }
      break;
    case CLASS_FTRACKBAR:
      switch(Key){
        case GLUT_KEY_PAGE_DOWN:
          u*=10;
        case GLUT_KEY_LEFT:
          Item->Data.FTrackBar->Position-=u*Item->Data.FTrackBar->Increment;
          if (Item->Data.FTrackBar->Position<Item->Data.FTrackBar->Min)
            Item->Data.FTrackBar->Position=Item->Data.FTrackBar->Min;
          if (Item->Data.FTrackBar->OnChange)
            Item->Data.FTrackBar->OnChange(Item->Data.FTrackBar->Position);
          break;
        case GLUT_KEY_PAGE_UP:
          u*=10;
        case GLUT_KEY_RIGHT:
          Item->Data.FTrackBar->Position+=u*Item->Data.FTrackBar->Increment;
          if (Item->Data.FTrackBar->Position>Item->Data.FTrackBar->Max)
            Item->Data.FTrackBar->Position=Item->Data.FTrackBar->Max;
          if (Item->Data.FTrackBar->OnChange)
            Item->Data.FTrackBar->OnChange(Item->Data.FTrackBar->Position);
          break;
      }
      break;
  }
}

/*********************************************************************
 * MENU CALLBACKS                                                    *
 *********************************************************************/

void menuDisplay(menu *Menu,int Width,int Height){
  int i;
  if (!Menu->Active)
    return;
  glPushAttrib(GL_ALL_ATTRIB_BITS);
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0,Width,Height,0);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
//  glEnable(GL_LINE_SMOOTH);
  paintMenuPage(Menu->Pages[Menu->PageIndex],Width,Height,Menu->PageIndex,Menu->PageIndex<Menu->PageCount-1);
  glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glPopAttrib();
}

void menuKeyboard(menu *Menu,unsigned char Key){
  if (Key==27){
    Menu->Active=!Menu->Active;
    return;
  }
  if (!Menu->Active)
    return;
  if (Menu->Pages[Menu->PageIndex]->ItemIndex){
    menuitem *Item=Menu->Pages[Menu->PageIndex]->Items[Menu->Pages[Menu->PageIndex]->ItemIndex-1];
    if (!Item->Enabled)
      return;
    menuItemKeyboard(Item,Key);
  }
}

void menuSpecial(menu *Menu,int Key){
  int i;
  if (!Menu->Active)
    return;
  switch (Key){
    case GLUT_KEY_LEFT:
      if (!Menu->Pages[Menu->PageIndex]->ItemIndex && Menu->PageIndex){
        Menu->PageIndex--;
        return;
      }
      break;
    case GLUT_KEY_RIGHT:
      if (!Menu->Pages[Menu->PageIndex]->ItemIndex && Menu->PageIndex<Menu->PageCount-1){
        Menu->PageIndex++;
        return;
      }
      break;
    case GLUT_KEY_DOWN:
      for (i=Menu->Pages[Menu->PageIndex]->ItemIndex;i<Menu->Pages[Menu->PageIndex]->ItemCount;i++)
        if (Menu->Pages[Menu->PageIndex]->Items[i]->Enabled){
          Menu->Pages[Menu->PageIndex]->ItemIndex=i+1;
          break;
        }
      return;
    case GLUT_KEY_UP:
      for (i=Menu->Pages[Menu->PageIndex]->ItemIndex-2;i>=-1;i--)
        if (i==-1 || Menu->Pages[Menu->PageIndex]->Items[i]->Enabled){
          Menu->Pages[Menu->PageIndex]->ItemIndex=i+1;
          break;
        }
      return;
  }
  if (Menu->Pages[Menu->PageIndex]->ItemIndex){
    menuitem *Item=Menu->Pages[Menu->PageIndex]->Items[Menu->Pages[Menu->PageIndex]->ItemIndex-1];
    if (!Item->Enabled)
      return;
    menuItemSpecial(Item,Key);
  }
}

